﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;

/// @author  Antti-Jussi Lakanen, Vesa Lappalainen
/// @version 2025
/// 
/// <summary>
/// Demo 4, Bonus 1-2.
/// Piirretään paljon palloja ja tönitään niitä
/// </summary>
public class Pallot : PhysicsGame
{
    /// <summary>
    /// Luodaan ja lisätään ruudulle annetun kokoinen pallo
    /// annettuun paikkaan.
    /// </summary>
    /// <param name="peli">Peli, johon neliö piirretään</param>
    /// <param name="x">Kappaleen keskipisteen x-koordinaatti</param>
    /// <param name="y">Kappaleen keskipisteen y-koordinaatti</param>
    /// <param name="r">Pallon säde</param>
    /// <param name="vari">Pallon väri</param>
    public static PhysicsObject PiirraPallo(Game peli, double x, double y, double r, Color vari)
    {
        PhysicsObject kappale = new PhysicsObject(2 * r, 2 * r, Shape.Ellipse);
        kappale.Position = new Vector(x, y);
        kappale.Color = vari;
        peli.Add(kappale);
        return kappale;
    }

    /// <summary>
    /// Reunat, zoomaus pllaojen luonti ja näppäinten kuuntelu
    /// </summary>
    public override void Begin()
    {
        Level.Size = new Vector(800, 600); // asettaa pelialueen koon
        SetWindowSize(800, 600); // asettaa ikkunan koon
        Level.CreateBorders(); // luodaan kentälle reunat
        Camera.ZoomToLevel(); // zoomaa kameran siten että vain pelialue on näkyvillä, eikä muuta

        Level.Background.Color = Color.Black;
        Gravity = new Vector(0, 0);

        for (int i = 0; i < 100; i++)
        {
            PiirraSuorakulmio(
                this,
                RandomGen.NextDouble(Level.Left, Level.Right), // Kentän vasen ja oikea reuna
                RandomGen.NextDouble(Level.Bottom, Level.Top), // alareuna, yläreuna
                RandomGen.NextDouble(5, 50), // leveys
                RandomGen.NextDouble(5, 50) // korkeus
                );
        }

        for (int i = 0; i < 30; i++)
        {
            PiirraPallo(
                this,
                RandomGen.NextDouble(Level.Left, Level.Right), // Kentän vasen ja oikea reuna
                RandomGen.NextDouble(Level.Bottom, Level.Top), // alareuna, yläreuna
                RandomGen.NextDouble(5, 30) // leveys
                );
        }

        PhysicsObject isoPallo = new PhysicsObject(80, 80, Shape.Circle);
        Add(isoPallo);

        Keyboard.Listen(Key.Up, ButtonState.Pressed, LiikutaPalloa, "", isoPallo, new Vector(0, 1000));
        Keyboard.Listen(Key.Down, ButtonState.Pressed, LiikutaPalloa, "", isoPallo, new Vector(0, -1000));
        Keyboard.Listen(Key.Left, ButtonState.Pressed, LiikutaPalloa, "", isoPallo, new Vector(-1000, 0));
        Keyboard.Listen(Key.Right, ButtonState.Pressed, LiikutaPalloa, "", isoPallo, new Vector(1000, 0));

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        AddCollisionHandler(isoPallo, "vihollinen", PalloTormasi); // Guru tehtävää varten oman pallon ja muiden törmäys
        isoPallo.Image = LoadImage("King");

        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");

    }

    /// <summary>
    /// Annetaan pallolle isku pyydettyyn suuntaan ja pyydetyllä voimalla
    /// </summary>
    /// <param name="pallo">pallo jota lyödään</param>
    /// <param name="suunta">suunta/voimavektori lyöntiä varten</param>
    private void LiikutaPalloa(PhysicsObject pallo, Vector suunta)
    {
        pallo.Hit(suunta);
    }


    /// <summary>
    /// Tekee yhden pallon
    /// </summary>
    /// <param name="peli">Peli</param>
    /// <param name="x">X</param>
    /// <param name="y">Y</param>
    /// <param name="r">Säde</param>
    public static void PiirraPallo(Game peli, double x, double y, double r)
    {
        PhysicsObject pallo = new PhysicsObject(2 * r, 2 * r);
        pallo.Position = new Vector(x, y);
        pallo.Shape = Shape.Circle;
        pallo.Color = RandomGen.NextColor();
        pallo.Tag = "vihollinen"; // Guru-tehtävä: jotta voidaan antaa tunniste törmäyksille
        peli.Add(pallo);
    }

    /// <summary>
    /// Luodaan ja lisätään ruudulle annetun kokoinen
    /// suorakulmio annettuun paikkaan.
    /// </summary>
    /// <param name="peli">Peli, johon neliö piirretään</param>
    /// <param name="x">Keskipisteen x-koordinaatti</param>
    /// <param name="y">Keskipisteen y-koordinaatti</param>
    /// <param name="w">Leveys</param>
    /// <param name="h">Korkeus.</param>
    public static void PiirraSuorakulmio(Game peli, double x, double y, double w, double h)
    {
        PhysicsObject kappale = new PhysicsObject(w, h, Shape.Rectangle);
        kappale.Position = new Vector(x, y);
        kappale.Color = RandomGen.NextColor();
        peli.Add(kappale);
    }

    /// <summary>
    /// Guru-tehtävää varten pallojen törmäysten käsittely
    /// </summary>
    /// <param name="tormaaja">Oma pallo</param>
    /// <param name="kohde">kohde johon törmättiin</param>
    private void PalloTormasi(PhysicsObject tormaaja, PhysicsObject kohde)
    {
        if (kohde.Y < 0) return;
        Explosion rajahdys = new Explosion(10);
        rajahdys.Sound = null; // Tämä laittaa äänet pois päältä. 
        rajahdys.Position = kohde.Position;
        Add(rajahdys);
        kohde.Destroy();
    }
}